
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;											;
;	The Great Race							;
;	A car game based on the pic16f648a uC	;
;											;
;	by brad slattery 2008					;
;											;
;	email:	brad@retrobrad.com				;
;	web:	www.retrobrad.com/electronics	;
;											;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


		LIST p=16F677 ;tell assembler what chip we are using
		include "P16F677.inc" ;include the defaults for the chip
		__config 0x0014 ;sets the configuration settings (oscillator type etc.)
		
	PC equ 0x02					;	The program counter will be refered to as PC

	cblock 0x20				;	start of general purpose registers - this is where we start defining variable names
		counta				;	used in the delay routines
		countb 				;	used in the delay routines
		countc
		pc_data
		vram_1				;	a video ram location
		vram_2				;	a video ram location
		vram_3				;	a video ram location
		vram_4				;	a video ram location
		vram_5				;	a video ram location
		vram_6				;	a video ram location
		vram_7				;	a video ram location
		vram_8				;	a video ram location
		vtemp_1				;	a temporary video ram location (used when tranfering vram data to the next vram location)
		vtemp_2				;	a temporary video ram location (used when tranfering vram data to the next vram location)
		vtemp_3				;	a temporary video ram location (used when tranfering vram data to the next vram location)
		vtemp_4				;	a temporary video ram location (used when tranfering vram data to the next vram location)
		vtemp_5				;	a temporary video ram location (used when tranfering vram data to the next vram location)
		vtemp_6				;	a temporary video ram location (used when tranfering vram data to the next vram location)
		vtemp_7				;	a temporary video ram location (used when tranfering vram data to the next vram location)
		repeat_frame		;	used to determine how many times we will be repeating each 'frame'
	endc					;	end of general purpose registers

	org 0x0000					;	org sets the origin, 0x0000 for the 16F628,

	bsf STATUS, RP0 			; select bank 1
	movlw b'00000000' 			; set PORTB all outputs
	movwf TRISB					;
	movlw b'00101000' 			; set PORTA all outputs except for bit 5 and 3
	movwf TRISA
	movlw b'00000000'
	movwf TRISC 				;
	bcf STATUS, RP0 			; select bank 0

	goto setup		; go straight to the setup routine

	text_data
	incf pc_data, 1	
	movf pc_data, 0	
	addwf PC
	nop
	retlw b'00111111'
	retlw b'01000000'
	retlw b'00111100'
	retlw b'01000000'
	retlw b'00111111'
	retlw b'00000000'
	retlw b'01111111'
	retlw b'01001001'
	retlw b'01001001'
	retlw b'01000001'
	retlw b'00000000'
	retlw b'01111111'
	retlw b'01001000'
	retlw b'01001100'
	retlw b'00110011'
	retlw b'00000000'
	retlw b'01111111'
	retlw b'01001000'
	retlw b'01001100'
	retlw b'00110011'
	retlw b'00000000'
	retlw b'01110010'
	retlw b'00001001'
	retlw b'00001001'
	retlw b'01111110'
	retlw b'00000000'
	retlw b'00000000'
	retlw b'00000000'
	retlw b'00111110'
	retlw b'01000001'
	retlw b'01000001'
	retlw b'00100010'
	retlw b'00000000'
	retlw b'01111111'
	retlw b'00001000'
	retlw b'00001000'
	retlw b'01111111'
	retlw b'00000000'
	retlw b'01111111'
	retlw b'01001000'
	retlw b'01001100'
	retlw b'00110011'
	retlw b'00000000'
	retlw b'01000001'
	retlw b'01000001'
	retlw b'01111111'
	retlw b'01000001'
	retlw b'01000001'
	retlw b'00000000'
	retlw b'00110010'
	retlw b'01001001'
	retlw b'01001001'
	retlw b'00100110'
	retlw b'00000000'
	retlw b'01000000'
	retlw b'01000000'
	retlw b'01111111'
	retlw b'01000000'
	retlw b'01000000'
	retlw b'00000000'
	retlw b'00111111'
	retlw b'01000000'
	retlw b'00111100'
	retlw b'01000000'
	retlw b'00111111'
	retlw b'00000000'
	retlw b'00011111'
	retlw b'00100100'
	retlw b'01000100'
	retlw b'00100100'
	retlw b'00011111'
	retlw b'00000000'
	retlw b'00110010'
	retlw b'01001001'
	retlw b'01001001'
	retlw b'00100110'
	retlw b'00000000'
	retlw b'00000000'
	retlw b'00000000'
	retlw b'00000000'
	retlw b'00000000'
	retlw b'00000000'
	retlw b'00000000'
	retlw b'00000000'
	retlw b'00000000'
	movlw h'01'
	movwf pc_data
	retlw b'00111111'			;this needs to be the same as the very first byte of data


setup						;	Okay, lets get everything setup, ready to play the game!
	movlw b'11111111'		; turn all cathodes OFF
	movwf PORTA
	movwf PORTB
	clrf pc_data
	clrf vram_1				;	these next lines clear our video ram.
	clrf vram_2				;	if we didnt clear them, then we would get random
	clrf vram_3				;	dots on our screen at startup and would more than likely crash
	clrf vram_4				;	as soon as we started playing! (experiment with not
	clrf vram_5				;	clearing all these vrams and see what happens!)
	clrf vram_6				;	....
	clrf vram_7				;	....
	clrf vram_8				;	....

begin							;	Our main program!
	call display				;	call the display routine
	call fill_vram				;	call the fill video ram routine
	goto begin					;	do it all again!

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;																	;
;	This next routine basically takes care of drawing the actual	;
;	graphics on the screen. You will notice that there are eight 	;
;	"sections" here and they basically look the same except for		;
;	minor differences. Thats because they are all doing the same	;
;	thing - activating a certain row of cathodes and then grabbing	;
;	the anode data in order to turn the leds on. The only			;
;	difference between them is that each section activates a		;
;	different row of cathodes and also grabs it's data to be		;
;	displayed from a different vram location.						;
;																	;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

display	
				movlw d'07'
				movwf repeat_frame				;	so we grab that data from speed and copy it to our frame_rate
loop
				movlw b'11111110'
				movwf PORTA
				movf vram_1, w					;	Now we grab the first COLUMN of data from vram_1.
				movwf PORTC						;	then we copy it to PORTB
				call delay
				clrf PORTC
					movlw b'11111111'
					movwf PORTA
					movwf PORTB
										
				movlw b'11111101'
				movwf PORTA
				movf vram_2, w					;	Now we grab the first COLUMN of data from vram_1.
				movwf PORTC						;	then we copy it to PORTB
				call delay
				clrf PORTC
					movlw b'11111111'
					movwf PORTA
					movwf PORTB
										
				movlw b'11111011'
				movwf PORTA
				movf vram_3, w					;	Now we grab the first COLUMN of data from vram_1.
				movwf PORTC						;	then we copy it to PORTB
				call delay
				clrf PORTC
					movlw b'11111111'
					movwf PORTA
					movwf PORTB
										
				movlw b'11101111'
				movwf PORTA
				movf vram_4, w					;	Now we grab the first COLUMN of data from vram_1.
				movwf PORTC						;	then we copy it to PORTB
				call delay
				clrf PORTC
					movlw b'11111111'
					movwf PORTA
					movwf PORTB
										
				movlw b'11101111'
				movwf PORTB
				movf vram_5, w					;	Now we grab the first COLUMN of data from vram_1.
				movwf PORTC						;	then we copy it to PORTB
				call delay
				clrf PORTC
					movlw b'11111111'
					movwf PORTA
					movwf PORTB
										
				movlw b'11011111'
				movwf PORTB
				movf vram_6, w					;	Now we grab the first COLUMN of data from vram_1.
				movwf PORTC						;	then we copy it to PORTB
				call delay
				clrf PORTC
					movlw b'11111111'
					movwf PORTA
					movwf PORTB
										
				movlw b'10111111'
				movwf PORTB
				movf vram_7, w					;	Now we grab the first COLUMN of data from vram_1.
				movwf PORTC						;	then we copy it to PORTB
				call delay
				clrf PORTC
					movlw b'11111111'
					movwf PORTA
					movwf PORTB
										
				movlw b'01111111'
				movwf PORTB
				movf vram_8, w					;	Now we grab the first COLUMN of data from vram_1.
				movwf PORTC						;	then we copy it to PORTB
				call delay
				clrf PORTC
					movlw b'11111111'
					movwf PORTA
					movwf PORTB
					decfsz repeat_frame							;	then decrease repeat_frame by one, 
		goto loop								;	if its not zero then draw it all again!
	return										;	if it is zero, then go back to where we came from...


fill_vram
		movf vram_1, 0		;	Copy vram_1 into vtemp_1 to use as a backup
		movwf vtemp_1		;	storage location. (just before we overwrite vram_1)
		movf vram_2, 0		;	Copy vram_1 into vtemp_2 to use as a backup
		movwf vtemp_2		;	storage location. (just before we overwrite vram_2)
		movf vram_3, 0		;	Copy vram_1 into vtemp_3 to use as a backup
		movwf vtemp_3		;	storage location. (just before we overwrite vram_3)
		movf vram_4, 0		;	Copy vram_1 into vtemp_4 to use as a backup
		movwf vtemp_4		;	storage location. (just before we overwrite vram_4)
		movf vram_5, 0		;	Copy vram_1 into vtemp_5 to use as a backup
		movwf vtemp_5		;	storage location. (just before we overwrite vram_5)
		movf vram_6, 0		;	Copy vram_1 into vtemp_6 to use as a backup
		movwf vtemp_6		;	storage location. (just before we overwrite vram_6)
		movf vram_7, 0		;	Copy vram_7 into vtemp_7 to use as a backup
		movwf vtemp_7		;	storage location. (just before we overwrite vram_7)
		call text_data
		movwf vram_1		;	then copy the new byte of data to the first vram location
		movf vtemp_1, 0		;	copy what was in vram_1 into vram_2 (remember how we made
		movwf vram_2		;	a backup of vram_1 and called it vtemp_1.)
		movf vtemp_2, 0		;	copy what was in vram_2 into vram_3 (remember how we made
		movwf vram_3		;	a backup of vram_2 and called it vtemp_2.)
		movf vtemp_3, 0		;	copy what was in vram_3 into vram_4 (remember how we made
		movwf vram_4		;	a backup of vram_3 and called it vtemp_3.)
		movf vtemp_4, 0		;	copy what was in vram_4 into vram_5 (remember how we made
		movwf vram_5		;	a backup of vram_4 and called it vtemp_4.)
		movf vtemp_5, 0		;	copy what was in vram_5 into vram_6 (remember how we made
		movwf vram_6		;	a backup of vram_5 and called it vtemp_5.)
		movf vtemp_6, 0		;	copy what was in vram_6 into vram_7 (remember how we made
		movwf vram_7		;	a backup of vram_6 and called it vtemp_6.)
		movf vtemp_7, 0		;	copy what was in vram_7 into vram_8 (remember how we made
		movwf vram_8		;	a backup of vram_7 and called it vtemp_7.)
	return					;	Now that we're all done here, go back to our main program.

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;																	;
;	Heres the delay routines!										;
;																	;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

delay									;	This first delay is a rather fast one.
			movlw d'03'					;	You can make the delay longer by
			movwf counta				;	increasing the decimal value. Or you
			movwf countb				;	can make the delay shorter by decreasing
again		decfsz counta, f				;	the decimal value.
		goto again					;
			decfsz countb, f			;
		goto again					;	once counta and countb have reached zero
	return								;	it will return to the main program


	end									;	That's it!